@@ -20,8 +20,8 @@
 
 #include "entryinternal.h"
 
-#include <QtCore/QStringList>
-#include <QtCore/QXmlStreamReader>
+#include <QStringList>
+#include <QXmlStreamReader>
 #include <QImage>
 #include <knewstuffcore_debug.h>
 
@@ -79,6 +79,7 @@ public:
     QString mProviderId;
     QStringList mUnInstalledFiles;
     QString mDonationLink;
+    QStringList mTags;
 
     QString mChecksum;
     QString mSignature;
@@ -156,6 +157,16 @@ void EntryInternal::setProviderId(const QString &id)
     d->mProviderId = id;
 }
 
+QStringList KNSCore::EntryInternal::tags() const
+{
+    return d->mTags;
+}
+
+void KNSCore::EntryInternal::setTags(const QStringList &tags)
+{
+    d->mTags = tags;
+}
+
 QString EntryInternal::category() const
 {
     return d->mCategory;
@@ -481,7 +492,7 @@ static int readInt(QXmlStreamReader* xml)
 bool KNSCore::EntryInternal::setEntryXML(QXmlStreamReader& reader)
 {
     if (reader.name() != QLatin1String("stuff")) {
-        qWarning() << "Parsing Entry from invalid XML";
+        qCWarning(KNEWSTUFFCORE) << "Parsing Entry from invalid XML. Reader tag name was expected to be \"stuff\", but was found as:" << reader.name();
         return false;
     }
 
@@ -540,6 +551,8 @@ bool KNSCore::EntryInternal::setEntryXML(QXmlStreamReader& reader)
             d->mInstalledFiles.append(reader.readElementText(QXmlStreamReader::SkipChildElements));
         } else if (reader.name() == QLatin1String("id")) {
             d->mUniqueId = reader.readElementText(QXmlStreamReader::SkipChildElements);
+        } else if (reader.name() == QLatin1String("tags")) {
+            d->mTags = reader.readElementText(QXmlStreamReader::SkipChildElements).split(QLatin1Char(','));
         } else if (reader.name() == QLatin1String("status")) {
             const auto statusText = readText(&reader);
             if (statusText == QLatin1String("installed")) {
@@ -551,7 +564,7 @@ bool KNSCore::EntryInternal::setEntryXML(QXmlStreamReader& reader)
             if (reader.tokenType() == QXmlStreamReader::Characters)
                 readNextSkipComments(&reader);
         }
-        Q_ASSERT(reader.tokenType() == QXmlStreamReader::EndElement);
+        Q_ASSERT_X(reader.tokenType() == QXmlStreamReader::EndElement, Q_FUNC_INFO, QStringLiteral("token name was %1 and the type was %2").arg(reader.name().toString()).arg(reader.tokenString()).toLocal8Bit().data());
     }
 
     // Validation
@@ -633,6 +646,8 @@ bool KNSCore::EntryInternal::setEntryXML(const QDomElement &xmldata)
             d->mInstalledFiles.append(e.text());
         } else if (e.tagName() == QLatin1String("id")) {
             d->mUniqueId = e.text();
+        } else if (e.tagName() == QLatin1String("tags")) {
+            d->mTags = e.text().split(QLatin1Char(','));
         } else if (e.tagName() == QLatin1String("status")) {
             QString statusText = e.text();
             if (statusText == QLatin1String("installed")) {
@@ -709,7 +724,7 @@ QDomElement KNSCore::EntryInternal::entryXML() const
     if (!d->mChecksum.isEmpty()) {
         (void)addElement(doc, el, QStringLiteral("checksum"), d->mChecksum);
     }
-    foreach (const QString &file, d->mInstalledFiles) {
+    for (const QString &file : qAsConst(d->mInstalledFiles)) {
         (void)addElement(doc, el, QStringLiteral("installedfile"), file);
     }
     if (!d->mUniqueId.isEmpty()) {
@@ -724,6 +739,7 @@ QDomElement KNSCore::EntryInternal::entryXML() const
     e = addElement(doc, el, QStringLiteral("preview"), d->mPreviewUrl[PreviewSmall1]);
     e = addElement(doc, el, QStringLiteral("previewBig"), d->mPreviewUrl[PreviewBig1]);
     e = addElement(doc, el, QStringLiteral("payload"), d->mPayload);
+    e = addElement(doc, el, QStringLiteral("tags"), d->mTags.join(QLatin1Char(',')));
 
     if (d->mStatus == KNS3::Entry::Installed) {
         (void)addElement(doc, el, QStringLiteral("status"), QStringLiteral("installed"));
